
import java.util.HashMap;

public class NLInfixrel extends NLFormula {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  NLTerm T;
  String s;
  boolean isInWords = false;
  String translation = "";
  
  static final int DEFAULT = 1;
  static final int WORDS = 2;
  static final int POSITED = 4;
  static final int NEGATED = 8;
  static final int SINGULAR = 16;
  static final int PLURAL = 32;
  static final int VERBAL = 64;
  static final int PREPOSITIONAL = 128;
  
  static final int QUALIFIER_CODE = (WORDS|POSITED|SINGULAR|PREPOSITIONAL);
  static final int WORDS_SINGULAR_CODE = (WORDS|POSITED|SINGULAR|VERBAL);
  static final int WORDS_PLURAL_CODE = (WORDS|POSITED|PLURAL|VERBAL);
  static final int DEFAULT_CODE = (DEFAULT|POSITED|SINGULAR|VERBAL);
  
  boolean diag = false;
  
// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public NLInfixrel(NLTerm T, String s) {
    this.T = T; this.s = s;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public String xlat(int code) {
    String x = "";
    if (s!=null) {
        HashMap map = NLLookup.lookUp(s,"infix");
        String phrase = getPhrase(code,map);
        //diag:
        if (diag) {
          System.out.println(
            (String) map.get("symb") + "\n" +
            (String) map.get("nsym") + "\n" +
            (String) map.get("reln") + "\n" +
            (String) map.get("negn") + "\n" +
            (String) map.get("plur") + "\n" +
            (String) map.get("nplu") + "\n" +
            (String) map.get("prep")
          );
        }
        //
        if (phrase==null) {
          s = Evaluable.ddStrip(s);
          x = " $"+s+"$ ";
        } else {
          String[] d = {"",""};
          NLPhraseEvaluator ev = new NLPhraseEvaluator(phrase);
          x = ev.eval(d);
        }
    } else {
      x = T.xlat(code);
    }
    this.translation = x;
    return x;
  }
  
  public boolean isInWords() {
  //Should call xlat before this method.
    return isInWords;
  }

  public String getString() {
  //Should call xlat before this method.
    return translation;
  }

  String getPhrase(int code, HashMap map) {
    String phrase = "";
    //First determine if there is a symbolic phrase available.
    boolean symbolsAvailable = false;
    if ((code&POSITED)!=0) {
      phrase = (String) map.get(GrammarConstants.SYMB);
      symbolsAvailable = (phrase!=null);
    } else {
      phrase = (String) map.get(GrammarConstants.NSYM);
      symbolsAvailable = (phrase!=null);
    }
    //At this point phrase is already set up, provided we want
    //symbols.
    if ((code&WORDS)!=0 || !symbolsAvailable) {
    //We want words.
      this.isInWords = true;
      if ((code&PREPOSITIONAL)!=0) {
      //We want the prepositional form.
        phrase = (String) map.get(GrammarConstants.PREP);
      } else {
      //We want a verb form.
        switch (code) {
          case WORDS|VERBAL|POSITED|SINGULAR:
            phrase = (String) map.get(GrammarConstants.RELN);
          break;
          case WORDS|VERBAL|NEGATED|SINGULAR:
            phrase = (String) map.get(GrammarConstants.NEGN);
          break;
          case WORDS|VERBAL|POSITED|PLURAL:
            phrase = (String) map.get(GrammarConstants.PLUR);
          break;
          case WORDS|VERBAL|NEGATED|PLURAL:
            phrase = (String) map.get(GrammarConstants.NPLU);
          break;
        }
      }
    }
    return phrase;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 
