

import java.io.File;
import java.io.FileInputStream;
import java.io.FileFilter;
import java.io.IOException;
import java.util.regex.*;
import java.util.HashMap;
import java.util.Vector;

public class NLLookup {

// --------------------------------------------------------------------------
// Fields
// --------------------------------------------------------------------------

    static final String NLPDIR = "../NLPhrases/";
    static byte[] buf;
    static int inBuf;
    static String wholeShebang;
    
    private static boolean hasBeenInitialized = false;
    
// --------------------------------------------------------------------------
// Methods
// --------------------------------------------------------------------------

/*
    public static void init() {
      //Reads in the contents of the phrase file,
      //and initializes the hash map. For now I'm going to presume the
      //phrase file is small enough that we've got plenty of RAM to just
      //read it all in. Your average entry won't be as many characters as
      //even six full 80-char lines. So upper bound of 512 chars/entry is
      //more than safe. That means you can have 2048 entries before you even
      //need 1 MB. I think we're safe.
      try {
        File inFile = new File(NLPHRASES);
        FileInputStream in = new FileInputStream(inFile);
        buf = new byte[(int) inFile.length()];
        inBuf = in.read(buf);
        in.close();
        wholeShebang = new String(buf,0,inBuf);
        NLLookup.hasBeenInitialized = true;
      } catch (IOException e) {e.printStackTrace();}
    }
*/

    public static void init() {
      try {
        File nlpDir = new File(NLPDIR);
        File[] nlpFiles = nlpDir.listFiles( new FileFilter() {
          public boolean accept(File pathname) {
            String name = pathname.getName();
            int L = name.length();
            if (L < 5) {return false;}
            else {
              String ext = name.substring(L-4,L);
              if (ext.equals(".nlp")) {return true;}
              else {return false;}
            }
          }
        });
        FileInputStream in;
        wholeShebang = "";
        for (File F : nlpFiles) {
          in = new FileInputStream(F);
          buf = new byte[(int) F.length()];
          inBuf = in.read(buf);
          in.close();
          wholeShebang += new String(buf,0,inBuf);
        }
        NLLookup.hasBeenInitialized = true;
      } catch (IOException e) {e.printStackTrace();}
    }
        
    public static HashMap lookUp(String s, String a) {
    //Returns a hash map where the keys are the headings such as
    //reln, negn, prep, what have you; and the values are the phrases
    //headed by these headings, returned just as unparsed strings.
    //Use the parsePhrase method to parse one of these strings.
      if (!NLLookup.hasBeenInitialized) {init();}
      //preserve passed values.
      String passedS = s, passedA = a;
      HashMap pm = new HashMap(); //"pm" is for "phrase map"
      s = slashPad(s);
      String reg = s+":"+a+"@\\s*";
      Pattern pat = Pattern.compile(reg);
      Matcher mat = pat.matcher(wholeShebang);
      if (mat.find()) {
        String tail = wholeShebang.substring(
                        mat.end(),wholeShebang.length()
                      );
        reg = "(\\w+):([^@]+)@\\s*|(@)";
        pat = Pattern.compile(reg);
        mat = pat.matcher(tail);
        boolean done = false;
        String key, value;
        while (!done && mat.find()) {
          if (mat.group(3)!=null) {done = true;}
          else {
            key = mat.group(1); value = mat.group(2);
            pm.put(key,value);
          }
        }
      } else {
      //The requested symbol was not found in the dictionary of NL phrases.
      //We give diagnostic output.
        System.err.printf(
          "NLLookup could not find %s in the dictionary of NL phrases.%n",
          passedS+":"+passedA
        );
      }
      return pm;
    }
        
    static String slashPad(String s) {
    //Pads with backslashes as necessary in order to respect
    //the metacharacters of regexes.
      StringBuffer sb = new StringBuffer();
      char c;
      for (int i = 0; i < s.length(); i++) {
        c = s.charAt(i);
        if (c=='\\' || c=='^' || c=='-' || c=='$' || c=='|' ||
            c=='(' || c==')' || c=='[' || c==']' || c=='{' || c=='}' ||
            c=='?' || c=='*' || c=='+' || c=='.') {
          sb.append('\\');
        }
        sb.append(c);
      }
      return new String(sb);
    }
        
    
// --------------------------------------------------------------------------
} // -------------------------------------------------------------------
// --------------------------------------------------------------------------


