
import java.util.HashMap;

public class NLMultiRelation extends NLFormula {
    
// ----------------------------------------------------------------------------
// Constructor
// ----------------------------------------------------------------------------

  public NLMultiRelation(boolean asserted, String s, NLTerm[][] ttt) {
    this.asserted = asserted; rstr = s; args = ttt;
  }

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  boolean asserted;
  String rstr;
  NLTerm[][] args;
  String[][] argStrings;
  boolean forceWords = false;
  boolean isInWords = false;
  String translation = "";
  
  boolean diag = false;
  
  static final int DEFAULT = 1;
  static final int WORDS = 2;
  static final int POSITED = 4;
  static final int NEGATED = 8;
  static final int SINGULAR = 16;
  static final int PLURAL = 32;
  static final int VERBAL = 64;
  static final int PREPOSITIONAL = 128;

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public String xlat(int code) {
//diag:
  if (diag) {
    System.out.printf("Running xlat for NLMultiRelation.%n");
  }
//      
    //Impose the asserted condition
    if (asserted) {
      code |= POSITED;
      code &= (~NEGATED);
    } else {
      code |= NEGATED;
      code &= (~POSITED);
    }
    //Since this is a multi-relation, impose plurality.
    code |= PLURAL;
    code &= ~SINGULAR;
    
    //Proceed.
    String x = "";
    processArgs(code);
    String arity = getArity();
    HashMap map = NLLookup.lookUp(rstr,arity);
//diag:
  if (diag) {
    System.out.printf("Code is: %d%n",code);
  }
//
    String phrase = getPhrase(code,map);
        //diag:
        if (diag) {
          System.out.println(
            (String) map.get("symb") + "\n" +
            (String) map.get("nsym") + "\n" +
            (String) map.get("reln") + "\n" +
            (String) map.get("negn") + "\n" +
            (String) map.get("plur") + "\n" +
            (String) map.get("nplu") + "\n" +
            (String) map.get("prep")
          );
        }
        //
    if (phrase==null) {
      x = makeDefaultPhrase(code);
    } else {
//diag:
  if (diag) {
    System.out.printf("Got the phrase we were looking for, which is:%n%s%n",
      phrase
    );
  }
//
      NLSubjectCombiner subc = new NLSubjectCombiner(phrase);
      x = subc.eval(argStrings);
    }
    this.translation = x;
    return x;
  }

  public boolean isInWords() {
  //Should call xlat before this method.
    return isInWords;
  }

  public String getString() {
  //Should call xlat before this method.
    return translation;
  }

  void processArgs(int code) {
    if (args!=null) {
      argStrings = new String[args.length][];
      for (int i = 0; i < args.length; i++) {
        if (args[i]!=null) {
          argStrings[i] = new String[args[i].length];
          for (int j = 0; j < args[i].length; j++) {
            argStrings[i][j] = args[i][j].xlat(NLInfixrel.DEFAULT_CODE);
            forceWords |= args[i][j].isInWords();
          }
        }
      }
    }
  }
  
  String getArity() {
  //We assume the arity is the length of args[0].
    String a = "0";
    if (args!=null) {
      a = Integer.toString(args[0].length);
    }
    return a;
  }
  
  String makeDefaultPhraseElement(int code, int i) {
  //processArgs should already have been called by this time.
  //For use when the symbol could not be found in the dictionary.
  //We construct the default phrase, which is just the symbol followed
  //by its arguments (if any), in the appropriate sort of braces; and
  //possibly preceded by a negation symbol.
    String r = rstr;
    if (argStrings[i]!=null) {
      r = ((code&NEGATED)!=0?"$\\neg$ ":"")+r;
      r += "[";
      for (int j = 0; j < argStrings[i].length; j++) {
        r += (j==0?"":",")+argStrings[i][j];
      }
      r += "]";
    }
    return "$"+r+"$";
  }
  
  String makeDefaultPhrase(int code) {
    String phrase = "";
    if (argStrings!=null) {
      for (int i = 0; i < argStrings.length; i++) {
        phrase += (i>0?" and ":"")+makeDefaultPhraseElement(code,i);
      }
    }
    return phrase;
  }

  
  String getPhrase(int code, HashMap map) {
    String phrase = null;
    //forceWords should be set up by this time.
    if (forceWords) {
      code |= WORDS;
      code &= (~DEFAULT);
    }
    //Now proceed.
    //First determine if there is a symbolic phrase available.
    boolean symbolsAvailable = false;
    if ((code&POSITED)!=0) {
      phrase = (String) map.get(GrammarConstants.SYMB);
      symbolsAvailable = (phrase!=null);
    } else {
      phrase = (String) map.get(GrammarConstants.NSYM);
      symbolsAvailable = (phrase!=null);
    }
    //At this point phrase is already set up, provided we want
    //symbols.
    if ((code&WORDS)!=0 || !symbolsAvailable) {
    //We want words.
      this.isInWords = true;
      if ((code&PREPOSITIONAL)!=0) {
      //We want the prepositional form.
        phrase = (String) map.get(GrammarConstants.PREP);
      } else {
      //We want a verb form.
        if ((code&SINGULAR)!=0) {
        //We want singular.
          if ((code&POSITED)!=0) {
          //Posited.
            phrase = (String) map.get(GrammarConstants.RELN);
          } else {
          //Negated.
            phrase = (String) map.get(GrammarConstants.NEGN);
          }
        } else {
        //We want plural.
          if ((code&POSITED)!=0) {
          //Posited.
            phrase = (String) map.get(GrammarConstants.PLUR);
          } else {
          //Negated.
            phrase = (String) map.get(GrammarConstants.NPLU);
          }
        }
      }
    }
    return phrase;
  }

}




