
import java.util.regex.*;
import java.util.Vector;

public class NLPhraseEvaluator {
  
  Vector parts = new Vector();
  
  public NLPhraseEvaluator() {}
  
  public NLPhraseEvaluator(String phrase) {
    NLPhraseEvaluator ev = NLPhraseEvaluator.compile(phrase);
    this.parts = ev.parts;
  }
  
  public Evaluable[] getPartsArray() {
    Evaluable[] a = new Evaluable[0];
    a = (Evaluable[]) parts.toArray(a);
    return a;
  }
  
  public void addPart(Evaluable p) {
    parts.add(p);
  }
  
  public String eval(String[] vars) {
    String s = "";
    Evaluable[] e = new Evaluable[0];
    e = (Evaluable[]) parts.toArray(e);
    if (e!=null) {
      for (int i = 0; i < e.length; i++) {
        s += e[i].eval(vars);
      }
    }
    return s;
  }
  
  public static NLPhraseEvaluator compile(String s) {
      NLPhraseEvaluator maker = new NLPhraseEvaluator();
      Pattern pat = Pattern.compile(
         "([^#%\\\\]+|\\\\.)"+
        "|#(\\^)?(\\d+)"+
        "|(%((\\w+)\\?)?)"
      );
      Matcher mat = pat.matcher(s);
      Evaluable ev;
      boolean inSubject = false;
      String gramRole = null;
      while (mat.find()) {
        //First set the current grammatical role, according to how many times
        //we've seen % signs so far. (We expect to see precisely two of them.)
        gramRole = (inSubject?Evaluable.SUBJECT_ROLE:null);
        if (mat.group(4)!=null) {
          if (mat.group(5)==null) {
          //Time to toggle the inSubject boolean.
            inSubject = !inSubject;
          } else {
          //We've encountered a markup tag.
            ev = new Evaluable(Evaluable.TAG_TYPE);
            ev.tag = mat.group(6);
            ev.grammaticalRole = gramRole;
            maker.addPart(ev);
          }
        } else if (mat.group(1)!=null) {
        //In this case we just matched some text.
          ev = new Evaluable(Evaluable.TEXT_TYPE);
          ev.text = mat.group(1);
          ev.grammaticalRole = gramRole;
          maker.addPart(ev);
        } else if (mat.group(3)!=null) {
        //In this case we matched a variable.
          if (mat.group(2)!=null) {
          //In this case we want to strip the dollar signs out of whatever is
          //passed for this variable.
            ev = new Evaluable(Evaluable.STRIP_VAR_TYPE);
            ev.varNum = Integer.parseInt(mat.group(3));
            ev.grammaticalRole = gramRole;
            maker.addPart(ev);
          } else {
          //We don't strip out the dollar signs.
            ev = new Evaluable(Evaluable.VAR_TYPE);
            ev.varNum = Integer.parseInt(mat.group(3));
            ev.grammaticalRole = gramRole;
            maker.addPart(ev);
          }
        }
      }
      return maker;
  }

}
